# coding: utf-8

"""
Copyright 2016 SmartBear Software

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Ref: https://github.com/swagger-api/swagger-codegen
"""

from datetime import datetime
from datetime import date
from pprint import pformat
import re
import json

from ..utils import sanitize_for_serialization

# type hinting support
from typing import TYPE_CHECKING
from typing import List
from typing import Dict

if TYPE_CHECKING:
    from . import EscalationTarget

class ManualEscalationResponse(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self) -> None:
        """
        ManualEscalationResponse - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'social_media_normalized_message_id': 'str',
            'conversation_normalized_message_id': 'str',
            'escalation_target': 'EscalationTarget',
            'escalation_status': 'str'
        }

        self.attribute_map = {
            'social_media_normalized_message_id': 'socialMediaNormalizedMessageId',
            'conversation_normalized_message_id': 'conversationNormalizedMessageId',
            'escalation_target': 'escalationTarget',
            'escalation_status': 'escalationStatus'
        }

        self._social_media_normalized_message_id = None
        self._conversation_normalized_message_id = None
        self._escalation_target = None
        self._escalation_status = None

    @property
    def social_media_normalized_message_id(self) -> str:
        """
        Gets the social_media_normalized_message_id of this ManualEscalationResponse.
        The Id of the message that got escalated.

        :return: The social_media_normalized_message_id of this ManualEscalationResponse.
        :rtype: str
        """
        return self._social_media_normalized_message_id

    @social_media_normalized_message_id.setter
    def social_media_normalized_message_id(self, social_media_normalized_message_id: str) -> None:
        """
        Sets the social_media_normalized_message_id of this ManualEscalationResponse.
        The Id of the message that got escalated.

        :param social_media_normalized_message_id: The social_media_normalized_message_id of this ManualEscalationResponse.
        :type: str
        """
        

        self._social_media_normalized_message_id = social_media_normalized_message_id

    @property
    def conversation_normalized_message_id(self) -> str:
        """
        Gets the conversation_normalized_message_id of this ManualEscalationResponse.
        The ID of the message in the conversation.

        :return: The conversation_normalized_message_id of this ManualEscalationResponse.
        :rtype: str
        """
        return self._conversation_normalized_message_id

    @conversation_normalized_message_id.setter
    def conversation_normalized_message_id(self, conversation_normalized_message_id: str) -> None:
        """
        Sets the conversation_normalized_message_id of this ManualEscalationResponse.
        The ID of the message in the conversation.

        :param conversation_normalized_message_id: The conversation_normalized_message_id of this ManualEscalationResponse.
        :type: str
        """
        

        self._conversation_normalized_message_id = conversation_normalized_message_id

    @property
    def escalation_target(self) -> 'EscalationTarget':
        """
        Gets the escalation_target of this ManualEscalationResponse.
        The target integration configuration used for an social media message.

        :return: The escalation_target of this ManualEscalationResponse.
        :rtype: EscalationTarget
        """
        return self._escalation_target

    @escalation_target.setter
    def escalation_target(self, escalation_target: 'EscalationTarget') -> None:
        """
        Sets the escalation_target of this ManualEscalationResponse.
        The target integration configuration used for an social media message.

        :param escalation_target: The escalation_target of this ManualEscalationResponse.
        :type: EscalationTarget
        """
        

        self._escalation_target = escalation_target

    @property
    def escalation_status(self) -> str:
        """
        Gets the escalation_status of this ManualEscalationResponse.
        Escalation Status of the message.

        :return: The escalation_status of this ManualEscalationResponse.
        :rtype: str
        """
        return self._escalation_status

    @escalation_status.setter
    def escalation_status(self, escalation_status: str) -> None:
        """
        Sets the escalation_status of this ManualEscalationResponse.
        Escalation Status of the message.

        :param escalation_status: The escalation_status of this ManualEscalationResponse.
        :type: str
        """
        if isinstance(escalation_status, int):
            escalation_status = str(escalation_status)
        allowed_values = ["Unknown", "Manual", "Throttled", "PreviouslyEscalated", "NotEscalated", "Automatic"]
        if escalation_status.lower() not in map(str.lower, allowed_values):
            # print("Invalid value for escalation_status -> " + escalation_status)
            self._escalation_status = "outdated_sdk_version"
        else:
            self._escalation_status = escalation_status

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_json(self):
        """
        Returns the model as raw JSON
        """
        return json.dumps(sanitize_for_serialization(self.to_dict()))

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other

